/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.falcon.notification.service.event;

import org.apache.falcon.state.ID;
import org.apache.oozie.client.WorkflowJob;
import org.joda.time.DateTime;

/**
 * An event generated by {@link org.apache.falcon.notification.service.impl.JobCompletionService}
 * indicating completion of a Job.
 */
public class JobCompletedEvent extends Event {

    private WorkflowJob.Status status;
    private final ID callbackID;
    private DateTime endTime;

    public JobCompletedEvent(ID callbackID, WorkflowJob.Status jobStatus, DateTime end) {
        this.callbackID = callbackID;
        this.status = jobStatus;
        this.endTime = end;
        this.type = EventType.JOB_COMPLETED;
    }

    public WorkflowJob.Status getStatus() {
        return status;
    }

    @Override
    public ID getTarget() {
        return callbackID;
    }

    public DateTime getEndTime() {
        return endTime;
    }
}
